@@ -8,6 +8,8 @@ mod arraybuffer;
 mod bigint;
 mod boolean;
 mod buffer;
+#[cfg(all(feature = "chrono_date", feature = "napi5"))]
+mod date;
 mod either;
 mod external;
 mod function;
@@ -22,6 +24,7 @@ mod serde;
 mod string;
 mod symbol;
 mod task;
+mod value_ref;
 
 #[cfg(feature = "napi5")]
 pub use crate::JsDate as Date;
@@ -41,6 +44,7 @@ pub use promise::*;
 pub use string::*;
 pub use symbol::*;
 pub use task::*;
+pub use value_ref::*;
 
 #[cfg(feature = "latin1")]
 pub use string::latin1_string::*;
@@ -85,6 +89,10 @@ pub trait FromNapiValue: Sized {
   ///
   /// this function called to convert napi values to native rust values
   unsafe fn from_napi_value(env: sys::napi_env, napi_val: sys::napi_value) -> Result<Self>;
+
+  fn from_unknown(value: JsUnknown) -> Result<Self> {
+    unsafe { Self::from_napi_value(value.0.env, value.0.value) }
+  }
 }
 
 pub trait FromNapiRef {
@@ -112,10 +120,13 @@ pub trait ValidateNapiValue: FromNapiValue + TypeName {
   /// # Safety
   ///
   /// this function called to validate whether napi value passed to rust is valid type
-  unsafe fn validate(env: sys::napi_env, napi_val: sys::napi_value) -> Result<()> {
+  unsafe fn validate(
+    env: sys::napi_env,
+    napi_val: sys::napi_value,
+  ) -> Result<napi_sys::napi_value> {
     let available_types = Self::type_of();
     if available_types.is_empty() {
-      return Ok(());
+      return Ok(ptr::null_mut());
     }
 
     let mut result = -1;
@@ -126,7 +137,7 @@ pub trait ValidateNapiValue: FromNapiValue + TypeName {
 
     let received_type = ValueType::from(result);
     if available_types.contains(&received_type) {
-      Ok(())
+      Ok(ptr::null_mut())
     } else {
       Err(Error::new(
         Status::InvalidArg,
