@@ -1,10 +1,14 @@
 use std::any::TypeId;
 use std::convert::TryInto;
 use std::ffi::CString;
+#[cfg(all(feature = "tokio_rt", feature = "napi4"))]
+use std::future::Future;
 use std::mem;
 use std::os::raw::{c_char, c_void};
 use std::ptr;
 
+#[cfg(all(feature = "tokio_rt", feature = "napi4"))]
+use crate::bindgen_runtime::ToNapiValue;
 use crate::{
   async_work::{self, AsyncWorkPromise},
   check_status,
@@ -28,8 +32,6 @@ use crate::JsError;
 use serde::de::DeserializeOwned;
 #[cfg(all(feature = "serde-json"))]
 use serde::Serialize;
-#[cfg(all(feature = "tokio_rt", feature = "napi4"))]
-use std::future::Future;
 
 pub type Callback = unsafe extern "C" fn(sys::napi_env, sys::napi_callback_info) -> sys::napi_value;
 
@@ -236,8 +238,7 @@ impl Env {
   /// This API allocates a node::Buffer object. While this is still a fully-supported data structure, in most cases using a TypedArray will suffice.
   pub fn create_buffer(&self, length: usize) -> Result<JsBufferValue> {
     let mut raw_value = ptr::null_mut();
-    let mut data: Vec<u8> = Vec::with_capacity(length);
-    let mut data_ptr = data.as_mut_ptr() as *mut c_void;
+    let mut data_ptr = ptr::null_mut();
     check_status!(unsafe {
       sys::napi_create_buffer(self.0, length, &mut data_ptr, &mut raw_value)
     })?;
@@ -248,7 +249,7 @@ impl Env {
         value: raw_value,
         value_type: ValueType::Object,
       }),
-      mem::ManuallyDrop::new(data),
+      mem::ManuallyDrop::new(unsafe { Vec::from_raw_parts(data_ptr as *mut _, length, length) }),
     ))
   }
 
@@ -366,8 +367,7 @@ impl Env {
 
   pub fn create_arraybuffer(&self, length: usize) -> Result<JsArrayBufferValue> {
     let mut raw_value = ptr::null_mut();
-    let mut data: Vec<u8> = Vec::with_capacity(length as usize);
-    let mut data_ptr = data.as_mut_ptr() as *mut c_void;
+    let mut data_ptr = ptr::null_mut();
     check_status!(unsafe {
       sys::napi_create_arraybuffer(self.0, length, &mut data_ptr, &mut raw_value)
     })?;
@@ -776,7 +776,7 @@ impl Env {
       } else {
         Err(Error::new(
           Status::InvalidArg,
-          "Invalid argument, T on unrwap is not the type of wrapped object".to_owned(),
+          "Invalid argument, T on unwrap is not the type of wrapped object".to_owned(),
         ))
       }
     }
@@ -803,7 +803,7 @@ impl Env {
       } else {
         Err(Error::new(
           Status::InvalidArg,
-          "Invalid argument, T on unrwap is not the type of wrapped object".to_owned(),
+          "Invalid argument, T on unwrap is not the type of wrapped object".to_owned(),
         ))
       }
     }
@@ -824,7 +824,7 @@ impl Env {
       } else {
         Err(Error::new(
           Status::InvalidArg,
-          "Invalid argument, T on unrwap is not the type of wrapped object".to_owned(),
+          "Invalid argument, T on unwrap is not the type of wrapped object".to_owned(),
         ))
       }
     }
@@ -1040,7 +1040,7 @@ impl Env {
   #[cfg(feature = "napi4")]
   pub fn create_threadsafe_function<
     T: Send,
-    V: NapiRaw,
+    V: ToNapiValue,
     R: 'static + Send + FnMut(ThreadSafeCallContext<T>) -> Result<Vec<V>>,
   >(
     &self,
@@ -1054,7 +1054,7 @@ impl Env {
   #[cfg(all(feature = "tokio_rt", feature = "napi4"))]
   pub fn execute_tokio_future<
     T: 'static + Send,
-    V: 'static + NapiValue,
+    V: 'static + ToNapiValue,
     F: 'static + Send + Future<Output = Result<T>>,
     R: 'static + Send + Sync + FnOnce(&mut Env, T) -> Result<V>,
   >(
@@ -1065,7 +1065,7 @@ impl Env {
     use crate::tokio_runtime;
 
     let promise = tokio_runtime::execute_tokio_future(self.0, fut, |env, val| unsafe {
-      resolver(&mut Env::from_raw(env), val).map(|v| v.raw())
+      resolver(&mut Env::from_raw(env), val).and_then(|v| ToNapiValue::to_napi_value(env, v))
     })?;
 
     Ok(unsafe { JsObject::from_raw_unchecked(self.0, promise) })
@@ -1143,7 +1143,7 @@ impl Env {
       } else {
         Err(Error::new(
           Status::InvalidArg,
-          "Invalid argument, T on unrwap is not the type of wrapped object".to_owned(),
+          "Invalid argument, T on unwrap is not the type of wrapped object".to_owned(),
         ))
       }
     }
