@@ -1,11 +1,27 @@
 use std::ptr;
 
 use super::Value;
-use crate::check_status;
+use crate::bindgen_runtime::TypeName;
+#[cfg(feature = "napi4")]
+use crate::{
+  bindgen_runtime::ToNapiValue,
+  threadsafe_function::{ThreadSafeCallContext, ThreadsafeFunction},
+};
+use crate::{check_status, ValueType};
 use crate::{sys, Env, Error, JsObject, JsUnknown, NapiRaw, NapiValue, Result, Status};
 
 pub struct JsFunction(pub(crate) Value);
 
+impl TypeName for JsFunction {
+  fn type_name() -> &'static str {
+    "Function"
+  }
+
+  fn value_type() -> crate::ValueType {
+    ValueType::Function
+  }
+}
+
 /// See [Working with JavaScript Functions](https://nodejs.org/api/n-api.html#n_api_working_with_javascript_functions).
 ///
 /// Example:
@@ -22,7 +38,6 @@ pub struct JsFunction(pub(crate) Value);
 /// ```
 impl JsFunction {
   /// [napi_call_function](https://nodejs.org/api/n-api.html#n_api_napi_call_function)
-  #[inline]
   pub fn call<V>(&self, this: Option<&JsObject>, args: &[V]) -> Result<JsUnknown>
   where
     V: NapiRaw,
@@ -57,7 +72,6 @@ impl JsFunction {
 
   /// [napi_call_function](https://nodejs.org/api/n-api.html#n_api_napi_call_function)
   /// The same with `call`, but without arguments
-  #[inline]
   pub fn call_without_args(&self, this: Option<&JsObject>) -> Result<JsUnknown> {
     let raw_this = this
       .map(|v| unsafe { v.raw() })
@@ -83,12 +97,10 @@ impl JsFunction {
     unsafe { JsUnknown::from_raw(self.0.env, return_value) }
   }
 
-  /// https://nodejs.org/api/n-api.html#n_api_napi_new_instance
+  /// <https://nodejs.org/api/n-api.html#n_api_napi_new_instance>
   ///
   /// This method is used to instantiate a new `JavaScript` value using a given `JsFunction` that represents the constructor for the object.
-  #[allow(clippy::new_ret_no_self)]
-  #[inline]
-  pub fn new<V>(&self, args: &[V]) -> Result<JsObject>
+  pub fn new_instance<V>(&self, args: &[V]) -> Result<JsObject>
   where
     V: NapiRaw,
   {
@@ -109,4 +121,19 @@ impl JsFunction {
     })?;
     Ok(unsafe { JsObject::from_raw_unchecked(self.0.env, js_instance) })
   }
+
+  #[cfg(feature = "napi4")]
+  pub fn create_threadsafe_function<T, V, F, ES>(
+    &self,
+    max_queue_size: usize,
+    callback: F,
+  ) -> Result<ThreadsafeFunction<T, ES>>
+  where
+    T: 'static,
+    V: ToNapiValue,
+    F: 'static + Send + FnMut(ThreadSafeCallContext<T>) -> Result<Vec<V>>,
+    ES: crate::threadsafe_function::ErrorStrategy::T,
+  {
+    ThreadsafeFunction::create(self.0.env, self.0.value, max_queue_size, callback)
+  }
 }
